// This is a -*- C++ -*- header.
//========================================================================
// last updated : [2009/01/06] -18:55:09(JST)-
//  written by $Author: okamura $ on $Date: 2009/01/06 10:17:55 $.
// -----------------------------------------------------------------------
// $Name:  $-$Revision: 4.2 $-$State: Exp $
// $Id: ne.res.cc,v 4.2 2009/01/06 10:17:55 okamura Exp $
//========================================================================
// -----------------------------------------------------------------------
//  COMMENT
// ---------
//  fit function for the nuclear resonance event for nu_e
//-----------------------------------------------------------------------
#include <cmath>
//! definition of \f$  \sqrt{2\pi}  \f$ for easy calculate
#define SQRT2PI   (2.5066282746310005024157652848110452530069867406099)

//---------------------------
//! \short calculate the deviation : \f$ \tilde{\sigma}_1^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \tilde{\sigma}_1^e (E_\nu) \f$ (MeV)
//---------------------------
double s1neres4(double E0)
{
    return 
        80.7509
        +0.0160873*E0
        -4.96623e-07*E0*E0;
}

//---------------------------
//! \short calculate the energy shift : 
//! \f$ \delta \tilde{E}_1^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta \tilde{E}_1^e (E_\nu) \f$ (MeV)
//---------------------------
double E1neres4(double E0)
{
    double s1=s1neres4(E0);
    return s1*(4.26266-0.000629255*E0+5.28187e-08*E0*E0);
}


//---------------------------
//! \short calculate the deviation : \f$ \tilde{\sigma}_2^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \tilde{\sigma}_2^e (E_\nu) \f$ (MeV)
//---------------------------
double s2neres4(double E0)
{
    return
        1370.22
        -104.195*sqrt(E0)
        +3.21074*E0
        -0.0422518*E0*sqrt(E0)
        +0.000207021*E0*E0;
}

//---------------------------
//! \short calculate the energy shift : 
//! \f$ \delta \tilde{E}_2^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta \tilde{E}_2^e (E_\nu) \f$ (MeV)
//---------------------------
double E2neres4(double E0)
{
    double s2=s2neres4(E0);
    return s2*
        (-9.993
         +1.18845*sqrt(E0)
         -0.0374465*E0
         +0.000491372*E0*sqrt(E0)
         -2.34903e-06*E0*E0);
}

//---------------------------
//! \short calculate the ratio : \f$ \tilde{r}_2^e (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return ratio : \f$ \tilde{r}_2^e (E_\nu) \f$
//---------------------------
double r2neres4(double E0)
{
    return 2.53527
        - 0.00286347*E0
        + 1.28846e-06*E0*E0
        - 2.435e-10*E0*E0*E0
        + 1.68505e-14*E0*E0*E0*E0;
}

//---------------------------
//! \short calculate the deviation : \f$ \tilde{\sigma}_3^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \tilde{\sigma}_3^e (E_\nu) \f$ (MeV)
//---------------------------
double s3neres4(double E0)
{
    return -2664.91
        +145.538*sqrt(E0)
        -2.31032*E0
        +0.0124255*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the energy shift : 
//! \f$ \delta \tilde{E}_3^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta \tilde{E}_3^e (E_\nu) \f$ (MeV)
//---------------------------
double E3neres4(double E0)
{
    double s3=s3neres4(E0);
    return 
        s3*(1670.27/(E0-932.079)+0.285849+0.000265052*E0);
}

//---------------------------
//! \short calculate the ratio : \f$ \tilde{r}_3^e (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return ratio : \f$ \tilde{r}_3^e (E_\nu) \f$
//---------------------------
double r3neres4(double E0)
{
    return -0.555356
        +0.03857*sqrt(E0)
        -0.000634634*E0
        +3.30267e-06*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the deviation : \f$ \tilde{\sigma}_4^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \tilde{\sigma}_4^e (E_\nu) \f$ (MeV)
//---------------------------
double s4neres4(double E0)
{
    return 12766.6
        -741.078*sqrt(E0)
        +13.8641*E0
        -0.0797288*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the energy shift : 
//! \f$ \delta \tilde{E}_4^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta \tilde{E}_4^e (E_\nu) \f$ (MeV)
//---------------------------
double E4neres4(double E0)
{
    return -1726.46
        +102.309*sqrt(E0)
        -0.687211*E0;
}

//---------------------------
//! \short calculate the ratio : \f$ \tilde{r}_4^e (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return ratio : \f$ \tilde{r}_4^e (E_\nu) \f$
//---------------------------
double r4neres4(double E0)
{
    return -0.463664
        +0.0261003*sqrt(E0)
        -0.000452045*E0
        +2.57522e-06*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the energy shift : 
//! \f$ \delta \hat{E}_1^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta \hat{E}_1^e (E_\nu) \f$ (MeV)
//---------------------------
double E1neres3(double E0)
{
    return 
        -4395.42
        +451.97*sqrt(E0)
        -14.1508*E0
        +0.146264*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the deviation : \f$ \hat{\sigma}_1^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \hat{\sigma}_1^e (E_\nu) \f$ (MeV)
//---------------------------
double s1neres3(double E0)
{
    return
        -1579.92
        +146.685*sqrt(E0)
        -4.2583*E0
        +0.0411491*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the energy shift : 
//! \f$ \delta \hat{E}_2^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta \hat{E}_2^e (E_\nu) \f$ (MeV)
//---------------------------
double E2neres3(double E0)
{
    return 
        -9192.95
        +801.025*sqrt(E0)
        -21.6297*E0
        +0.192016*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the deviation : \f$ \hat{\sigma}_2^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \hat{\sigma}_2^e (E_\nu) \f$ (MeV)
//---------------------------
double s2neres3(double E0)
{
    return
        -12.0281
        -0.11738*E0
        +0.000243597*E0*E0;
}

//---------------------------
//! \short calculate the ratio : \f$ \hat{r}_2^e (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return ratio : \f$ \hat{r}_2^e (E_\nu) \f$
//---------------------------
double r2neres3(double E0)
{
    return
        122.703
        -16.6668*sqrt(E0)
        +0.838695*E0
        -0.0185455*E0*sqrt(E0)
        +0.000152484*E0*E0;
}

//---------------------------
//! \short calculate the deviation : \f$ \hat{\sigma}_3^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \hat{\sigma}_3^e (E_\nu) \f$ (MeV)
//---------------------------
double s3neres3(double E0)
{
    return 
        -221.89
        +30.5464*sqrt(E0)
        -1.14164*E0
        +0.0134291*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the energy shift : 
//! \f$ \delta \hat{E}_3^e (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta \hat{E}_3^e (E_\nu) \f$ (MeV)
//---------------------------
double E3neres3(double E0)
{
    double s3=s3neres3(E0);
    return s3*
        (16.7635
         -0.0474977*E0
         +6.2183e-05*E0*E0
         -2.38464e-08*E0*E0*E0);
}

//---------------------------
//! \short calculate the ratio : \f$ \hat{r}_3^e (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return ratio : \f$ \hat{r}_3^e (E_\nu) \f$
//---------------------------
double r3neres3(double E0)
{
    return
        3.18725
        -0.00959426*E0
        +9.89507e-06*E0*E0
        -3.4213e-09*E0*E0*E0;
}

//---------------------------
//! \short calculate the normalization factor for 3 Gaussian: 
//! \f$ \hat{A}^e (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return normalization factor for 3 Gaussian:
//! \f$ \hat{A}^e (E_\nu) \f$
//---------------------------
double Aneres3(double E0)
{
    double s1=s1neres3(E0);

    double s2=s2neres3(E0);
    double r2=r2neres3(E0);

    double s3=s3neres3(E0);
    double r3=r3neres3(E0);

    return SQRT2PI*(s1 + s2*r2 + s3*r3);
}

//---------------------------
//! \short calculate the normalization factor for 4 Gaussian: 
//! \f$ \tilde{A}^e (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return normalization factor for 4 Gaussian:
//! \f$ \tilde{A}^e (E_\nu) \f$
//---------------------------
double Aneres4(double E0)
{
    double s1=s1neres4(E0);

    double s2=s2neres4(E0);
    double r2=r2neres4(E0);

    double s3=s3neres4(E0);
    double r3=r3neres4(E0);

    double s4=s4neres4(E0);
    double r4=r4neres4(E0);

    return SQRT2PI*(s1 + s2*r2 + s3*r3 + s4*r4);
}

/*!
 \short calculate the \f$ f_e^{\rm res;3G}(E_{\rm rec};E_\nu) \f$
 \param  En  : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
 \param  ene : reconstructed     energy, \f$ E_{\rm rec} \f$ (MeV)
 \return probability of the event distribution
 \f$ f_e^{\rm res;3G}(E_{\rm rec};E_\nu) \f$
*/
double Fneres3(double En,double ene)
{
    double E1=En-E1neres3(En);
    double E2=En-E2neres3(En);
    double E3=En-E3neres3(En);

    double s1=s1neres3(En);
    double s2=s2neres3(En);
    double s3=s3neres3(En);

    double r2=r2neres3(En);
    double r3=r3neres3(En);

    double a=Aneres3(En);

    double fitfc = 
        1.0/a*(1.0*exp(-((ene-E1)*(ene-E1))/(2.0*s1*s1))
               +r2*exp(-((ene-E2)*(ene-E2))/(2.0*s2*s2))
               +r3*exp(-((ene-E3)*(ene-E3))/(2.0*s3*s3)));

    return fitfc;

}

/*!
 \short calculate the \f$ f_e^{\rm res;4G}(E_{\rm rec};E_\nu) \f$
 \param  En  : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
 \param  ene : reconstructed     energy, \f$ E_{\rm rec} \f$ (MeV)
 \return probability of the event distribution
 \f$ f_e^{\rm res;4G}(E_{\rm rec};E_\nu) \f$
*/

double Fneres4(double En,double ene)
{
    double E1=En-E1neres4(En);
    double E2=En-E2neres4(En);
    double E3=En-E3neres4(En);
    double E4=En-E4neres4(En);

    double s1=s1neres4(En);
    double s2=s2neres4(En);
    double s3=s3neres4(En);
    double s4=s4neres4(En);

    double r2=r2neres4(En);
    double r3=r3neres4(En);
    double r4=r4neres4(En);

    double a=Aneres4(En);

    double fitfc = 
        1.0/a*(1.0*exp(-((ene-E1)*(ene-E1))/(2.0*s1*s1))
               +r2*exp(-((ene-E2)*(ene-E2))/(2.0*s2*s2))
               +r3*exp(-((ene-E3)*(ene-E3))/(2.0*s3*s3))
               +r4*exp(-((ene-E4)*(ene-E4))/(2.0*s4*s4)));

    return fitfc;

}

/*!
 \short calculate probability of the event distribution
 number of the Gaussian is changed automatically
 \param  En  : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
 \param  ene : reconstructed     energy, \f$ E_{\rm rec} \f$ (MeV)
 \return probability of the event distribution
*/
//---------------------------
double Fneres(double En,double ene)
{
    double fitfc=0.0;
    
    //-----------
    // 3g region 
    //-----------

    if(En<1200.0){
        fitfc=Fneres3(En,ene);
    }
    //-----------
    // 4g region
    //-----------
    if(En>=1200.0){
        fitfc=Fneres4(En,ene);
    }

    return fitfc;
}

