// This is a -*- C++ -*- header.
//========================================================================
// last updated : [2008/09/01] -18:50:50(JST)-
//  written by $Author: okamura $ on $Date: 2008/09/01 09:55:49 $.
// -----------------------------------------------------------------------
// $Name:  $-$Revision: 4.1 $-$State: Exp $
// $Id: nm.res.cc,v 4.1 2008/09/01 09:55:49 okamura Exp $
//========================================================================
// -----------------------------------------------------------------------
//  COMMENT
// ---------
//  fit function for the nuclear resonance event for nu_mu 
//-----------------------------------------------------------------------
#include <cmath>
//! definition of \f$  \sqrt{2\pi}  \f$ for easy calculate
#define SQRT2PI   (2.5066282746310005024157652848110452530069867406099)


//---------------------------
//! \short calculate the deviation : \f$ \tilde{\sigma}_1^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \tilde{\sigma}_1^\mu (E_\nu) \f$ (MeV)
//---------------------------
double s1nmres4(double E0)
{
    return 85.5061+0.00793854*E0+1.04033e-6*E0*E0;
}

//---------------------------
//! \short calculate the energy shift : 
//! \f$ \delta \tilde{E}_1^\mu (E_\nu) \f$ (Me)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta \tilde{E}_1^\mu (E_\nu) \f$ (MeV)
//---------------------------
double E1nmres4(double E0)
{
    double s1=s1nmres4(E0);
    return (4.25378 - 0.000499576*E0 + 2.57953e-08*E0*E0)*s1;
}

//---------------------------
//! \short calculate the deviation : \f$ \tilde{\sigma}_2^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \tilde{\sigma}_2^\mu (E_\nu) \f$ (MeV)
//---------------------------
double s2nmres4(double E0)
{
    return 98.8451 + 0.0263457*E0 + 2.37709e-06*E0*E0;
}

//---------------------------
//! \short calculate the energy shift :
//! \f$ \delta \tilde{E}_2^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta \tilde{E}_2^\mu (E_\nu) \f$ (MeV)
//---------------------------
double E2nmres4(double E0)
{
    double s2=s2nmres4(E0);
    return s2*(4.73574-0.000834175*E0+5.64171e-08*E0*E0);
}


//---------------------------
//! \short calculate the ratio : \f$ \tilde{r}_2^\mu (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return ratio : \f$ \tilde{r}_2^\mu (E_\nu) \f$
//---------------------------
double r2nmres4(double E0)
{
    return 2.54982
        -0.00268051*E0
        +1.15207e-06*E0*E0
        -2.15199e-10*E0*E0*E0
        +1.48889e-14*E0*E0*E0*E0;
}

//---------------------------
//! \short calculate the deviation : \f$ \tilde{\sigma}_3^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \tilde{\sigma}_3^\mu (E_\nu) \f$ (MeV)
//---------------------------
double s3nmres4(double E0)
{
    return -2182.81
        +111.425*sqrt(E0)
        -1.56793*E0
        +0.0073345*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the energy shift : 
//! \f$ \delta \tilde{E}_3^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta \tilde{E}_3^\mu (E_\nu) \f$ (MeV)
//---------------------------
double E3nmres4(double E0)
{
    double s3=s3nmres4(E0);
    return s3*(2102.76/(E0-913.991)-0.0525705+0.000346024*E0);
}

//---------------------------
//! \short calculate the ratio : \f$ \tilde{r}_3^\mu (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return ratio : \f$ \tilde{r}_3^\mu (E_\nu) \f$
//---------------------------
double r3nmres4(double E0)
{
    return 0.695009
        -0.02522*sqrt(E0)
        +0.000435572*E0
        -2.72825e-06*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the deviation : \f$ \tilde{\sigma}_4^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \tilde{\sigma}_4^\mu (E_\nu) \f$ (MeV)
//---------------------------
double s4nmres4(double E0)
{
    return 2717.63
        -172.039*sqrt(E0)
        +3.32363*E0
        -0.0167035*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the energy shift : 
//! \f$ \delta \tilde{E}_4^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta \tilde{E}_4^\mu (E_\nu) \f$ (MeV)
//---------------------------
double E4nmres4(double E0)
{
    return -508.748+1.7484*E0-0.0170093*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the ratio : \f$ \tilde{r}_4^\mu (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return ratio : \f$ \tilde{r}_4^\mu (E_\nu) \f$
//---------------------------
double r4nmres4(double E0)
{
    return 0.00408316*sqrt(E0)
        -0.00010848*E0
        +7.97081e-07*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the energy shift : 
//! \f$ \delta \hat{E}_1^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta \hat{E}_1^\mu (E_\nu) \f$ (MeV)
//---------------------------
double E1nmres3(double E0)
{
    return 
        - 4017.62 
        + 397.522*sqrt(E0)
        - 11.8366*E0
        +0.115918*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the deviation : \f$ \hat{\sigma}_1^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \hat{\sigma}_1^\mu (E_\nu) \f$ (MeV)
//---------------------------
double s1nmres3(double E0)
{
    return -2094.31 
        + 185.376*sqrt(E0) 
        - 5.21434*E0 
        + 0.0488352*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the energy shift : 
//! \f$ \delta \hat{E}_2^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta \hat{E}_2^\mu (E_\nu) \f$ (MeV)
//---------------------------
double E2nmres3(double E0)
{
    return -11783.7 + 1106.18*sqrt(E0)
        -33.4551*E0+0.342718*E0*sqrt(E0);
}

//---------------------------
//! \short calculate the deviation : \f$ \hat{\sigma}_2^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \hat{\sigma}_2^\mu (E_\nu) \f$ (MeV)
//---------------------------
double s2nmres3(double E0)
{
    return 211.566 - 0.618818*E0 + 0.000504858*E0*E0;
}

//---------------------------
//! \short calculate the ratio : \f$ \hat{r}_2^\mu (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return ratio : \f$ \hat{r}_2^\mu (E_\nu) \f$
//---------------------------
double r2nmres3(double E0)
{
    return -1.60359 + 0.00389364*E0 -1.86901e-06*E0*E0;
}

//---------------------------
//! \short calculate the deviation : \f$ \hat{\sigma}_3^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return deviation : \f$ \hat{\sigma}_3^\mu (E_\nu) \f$ (MeV)
//---------------------------
double s3nmres3(double E0)
{
    return - 514.979 
        + 2.64753*E0
        - 0.00465788*E0*E0
        +3.56231e-06*E0*E0*E0
        -1.01027e-09*E0*E0*E0*E0;
}

//---------------------------
//! \short calculate the energy shift : 
//! \f$ \delta \hat{E}_3^\mu (E_\nu) \f$ (MeV)
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return energy shift : \f$ \delta \hat{E}_3^\mu (E_\nu) \f$ (MeV)
//---------------------------
double E3nmres3(double E0)
{
    return 380.281 
        - 0.499613*E0
        + 0.000495517*E0*E0 
        - 1.6777e-07 *E0*E0*E0;
}

//---------------------------
//! \short calculate the ratio : \f$ \hat{r}_3^\mu (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return ratio : \f$ \hat{r}_3^\mu (E_\nu) \f$
//---------------------------
double r3nmres3(double E0)
{
    return 49.0181/(E0-507.231);
}

//---------------------------
//! \short calculate the normalization factor for 3 Gaussian : 
//! \f$ \hat{A}^\mu (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return normalization factor for 3 Gaussian:
//! \f$ \hat{A}^\mu (E_\nu) \f$
//---------------------------
double Anmres3(double E0)
{
    double s1=s1nmres3(E0);

    double s2=s2nmres3(E0);
    double r2=r2nmres3(E0);

    double s3=s3nmres3(E0);
    double r3=r3nmres3(E0);

    return SQRT2PI*(s1 + s2*r2 + s3*r3);
}

//---------------------------
//! \short calculate the normalization factor for 4 Gaussian : 
//! \f$ \tilde{A}^\mu (E_\nu) \f$
//! \param  E0 : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
//! \return normalization factor for 4 Gaussian:
//! \f$ \tilde{A}^\mu (E_\nu) \f$
//---------------------------
double Anmres4(double E0)
{
    double s1=s1nmres4(E0);

    double s2=s2nmres4(E0);
    double r2=r2nmres4(E0);

    double s3=s3nmres4(E0);
    double r3=r3nmres4(E0);

    double s4=s4nmres4(E0);
    double r4=r4nmres4(E0);

    return SQRT2PI*(s1 + s2*r2 + s3*r3 + s4*r4);
}

/*!
 \short calculate the \f$ f_\mu^{\rm res;3G}(E_{\rm rec};E_\nu) \f$
 \param  En  : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
 \param  ene : reconstructed     energy, \f$ E_{\rm rec} \f$ (MeV)
 \return probability of the event distribution
 \f$ f_\mu^{\rm res;3G}(E_{\rm rec};E_\nu) \f$
*/

double Fnmres3(double En,double ene)
{
    double E1=En-E1nmres3(En);
    double E2=En-E2nmres3(En);
    double E3=En-E3nmres3(En);

    double s1=s1nmres3(En);
    double s2=s2nmres3(En);
    double s3=s3nmres3(En);

    double r2=r2nmres3(En);
    double r3=r3nmres3(En);

    double a=Anmres3(En);

    double fitfc = 
        1.0/a*(1.0*exp(-((ene-E1)*(ene-E1))/(2.0*s1*s1))
               +r2*exp(-((ene-E2)*(ene-E2))/(2.0*s2*s2))
               +r3*exp(-((ene-E3)*(ene-E3))/(2.0*s3*s3)));

    return fitfc;

}

/*!
 \short calculate the \f$ f_\mu^{\rm res;4G}(E_{\rm rec};E_\nu) \f$
 \param  En  : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
 \param  ene : reconstructed     energy, \f$ E_{\rm rec} \f$ (MeV)
 \return probability of the event distribution
 \f$ f_\mu^{\rm res;4G}(E_{\rm rec};E_\nu) \f$
*/

double Fnmres4(double En,double ene)
{
    double E1=En-E1nmres4(En);
    double E2=En-E2nmres4(En);
    double E3=En-E3nmres4(En);
    double E4=En-E4nmres4(En);

    double s1=s1nmres4(En);
    double s2=s2nmres4(En);
    double s3=s3nmres4(En);
    double s4=s4nmres4(En);

    double r2=r2nmres4(En);
    double r3=r3nmres4(En);
    double r4=r4nmres4(En);

    double a=Anmres4(En);

    double fitfc = 
        1.0/a*(1.0*exp(-((ene-E1)*(ene-E1))/(2.0*s1*s1))
               +r2*exp(-((ene-E2)*(ene-E2))/(2.0*s2*s2))
               +r3*exp(-((ene-E3)*(ene-E3))/(2.0*s3*s3))
               +r4*exp(-((ene-E4)*(ene-E4))/(2.0*s4*s4)));

    return fitfc;

}

/*!
 \short calculate the probability of the event distribution
        the number of the Gaussian is changed automatically
 \param  En  : incoming neutrino energy, \f$ E_\nu \f$ (MeV)
 \param  ene : reconstructed     energy, \f$ E_{\rm rec} \f$ (MeV)
 \return probability of the event distribution
*/
//---------------------------
double Fnmres(double En,double ene)
{
    double fitfc=0.0;
    
    //-----------
    // 3g region 
    //-----------

    if(En<=1200.0){
        fitfc=Fnmres3(En,ene);
    }
    //-----------
    // 4g region
    //-----------
    if(En>1200.0){
        fitfc=Fnmres4(En,ene);
    }

    return fitfc;
}

